import unittest
import os
import sys
sys.path.insert(0, os.path.abspath(os.path.join(os.path.dirname(__file__), '..')))
import time
from pelops import mylogger, mqttclient, read_config
from alcathous.datapointmanager import DataPointManager
from alcathous.nodatabehavior import NoDataBehavior


# https://stackoverflow.com/questions/3173320/text-progress-bar-in-the-console
# Print iterations progress
def printProgressBar (iteration, total, prefix = '', suffix = '', decimals = 1, length = 100, fill = '█', msg=None):
    """
    Call in a loop to create terminal progress bar
    @params:
        iteration   - Required  : current iteration (Int)
        total       - Required  : total iterations (Int)
        prefix      - Optional  : prefix string (Str)
        suffix      - Optional  : suffix string (Str)
        decimals    - Optional  : positive number of decimals in percent complete (Int)
        length      - Optional  : character length of bar (Int)
        fill        - Optional  : bar fill character (Str)
    """
    percent = ("{0:." + str(decimals) + "f}").format(100 * (iteration / float(total)))
    filledLength = int(length * iteration // total)
    bar = fill * filledLength + '-' * (length - filledLength)
    if msg is None:
        print('\r%s |%s| %s%% %s' % (prefix, bar, percent, suffix), end = '\r')
    else:
        print('\r%s |%s| %s%% %s %s' % (prefix, bar, percent, suffix, msg), end='\r')
    # Print New Line on Complete
    if iteration == total:
        print()


class TestDatapointManager(unittest.TestCase):
    def _create_handler(self, id):
        self.logger.info("TestDatapointManager._create_handler - created handler for topic {}.".format(id))
        def _func(message):
            value = message.decode("utf-8")
            self.received_messages[id].append((time.time(), value))
        return _func

    def _add_handler(self, topic):
        _func = self._create_handler(topic)
        self.mqtt_client.subscribe(topic, _func)
        self.received_messages[topic] = []

    def init_handler(self, data_point_list):
        for dp in data_point_list:
            self._add_handler(dp._topic_sub)
            for m in dp.methods:
                self._add_handler(m._topic)

    @classmethod
    def setUpClass(cls):
        cls.config = read_config.read_config(
            os.path.abspath(os.path.join(os.path.dirname(__file__), '..'))+"/tests/config.yaml")
        cls.logger = mylogger.create_logger(cls.config["logger"], __name__)
        cls.logger.info("TestDatapointManager - start")

    @classmethod
    def tearDownClass(cls):
        cls.logger.info("TestDatapointManager - stop")

    def setUp(self):
        self.mqtt_client = mqttclient.MQTTClient(self.config["mqtt"], self.logger, True)
        self.mqtt_client.connect()
        self.received_messages = {}

    def tearDown(self):
        self.mqtt_client.disconnect()

    def test_dpm0_init(self):
        self.logger.info("TestDatapointManager.test_dpm_init ---------------------------------------------")
        dpm = DataPointManager(self.config, self.mqtt_client, self.logger)
        self.assertEqual(dpm._no_data_behavior, NoDataBehavior.LAST_VALID)
        self.assertEqual(dpm._update_cycle, 30)
        self.assertEqual(dpm._worker._number, 2)
        self.assertEqual(len(dpm._data_points), 2)
        self.assertEqual(len(dpm._purges), 2)
        self.assertEqual(len(dpm._processes), 7)
        process_names = []
        for dp in dpm._data_points:
            for m in dp.methods:
                process_names.append(m.name)
        process_names = sorted(process_names)
        self.assertListEqual(process_names, ['avg_5min', 'avg_5min', 'count_2min', 'max_3min', 'min_3min', 'wavg_5min', 'wavg_5min'])
        self.assertEqual(dpm._data_points[0]._topic_sub, "/test/0/raw")
        self.assertEqual(dpm._data_points[1]._topic_sub, "/test/1/raw")
        self.assertEqual(dpm._data_points[0]._topic_pub_prefix, "/test/0/aggregated/")
        self.assertEqual(dpm._data_points[1]._topic_pub_prefix, "/test/1/aggregated/")

    def test_dpm1_startstop(self):
        self.logger.info("TestDatapointManager.test_dpm_startstop ---------------------------------------------")
        dpm = DataPointManager(self.config, self.mqtt_client, self.logger)
        self.assertEqual(len(dpm._worker._list), 0)
        self.assertEqual(len(self.mqtt_client._topic_handler), 0)
        dpm.start()
        self.assertEqual(len(dpm._worker._list), 2)
        self.assertEqual(len(self.mqtt_client._topic_handler), 2)
        dpm.stop()
        self.assertEqual(len(dpm._worker._list), 0)
        self.assertEqual(len(self.mqtt_client._topic_handler), 0)

    def test_dpm2_full(self):
        self.logger.info("TestDatapointManager.test_dpm_full ---------------------------------------------")
        print("TestDatapointManager.test_dpm_full - this test takes approx. 12 minutes to complete.")
        start_time = time.time()
        dpm = DataPointManager(self.config, self.mqtt_client, self.logger)
        self.init_handler(dpm._data_points)
        dpm.start()
        time.sleep(5)
        printProgressBar(0, 72, prefix='Progress:', suffix='Complete', length=100)
        for value in range(0, 36):
            for dp in dpm._data_points:
                self.mqtt_client.publish(dp._topic_sub, value)
            time.sleep(10)
            diff = (time.time()-start_time)/60
            diff = "{:10.1f} min".format(diff)
            printProgressBar(value+1, 72, prefix='Progress:', suffix='Complete', length=100, msg=diff)
        for value in range(36, 72):
            time.sleep(10)
            diff = (time.time()-start_time)/60
            diff = "{:10.1f} min".format(diff)
            printProgressBar(value+1, 72, prefix='Progress:', suffix='Complete', length=100, msg=diff)
        dpm.stop()
        self.assertEqual(len(self.received_messages["/test/0/aggregated/wavg_5min"]), 25)
        self.assertEqual(len(self.received_messages["/test/1/aggregated/wavg_5min"]), 25)
        self.assertEqual(len(self.received_messages["/test/0/aggregated/avg_5min"]), 25)
        self.assertEqual(len(self.received_messages["/test/1/aggregated/avg_5min"]), 25)
        self.assertEqual(len(self.received_messages["/test/0/aggregated/count_2min"]), 25)
        self.assertEqual(len(self.received_messages["/test/0/aggregated/min_3min"]), 25)
        self.assertEqual(len(self.received_messages["/test/0/aggregated/max_3min"]), 25)

        self.assertAlmostEqual(float(self.received_messages["/test/0/aggregated/wavg_5min"][24][1]), 34.43, delta=0.1)
        self.assertAlmostEqual(float(self.received_messages["/test/1/aggregated/wavg_5min"][24][1]), 34.43, delta=0.1)
        self.assertAlmostEqual(float(self.received_messages["/test/0/aggregated/avg_5min"][24][1]), 34.0, delta=0.1)
        self.assertAlmostEqual(float(self.received_messages["/test/1/aggregated/avg_5min"][24][1]), 34.0, delta=0.1)
        self.assertAlmostEqual(float(self.received_messages["/test/0/aggregated/count_2min"][24][1]), 0, delta=0.1)
        self.assertAlmostEqual(float(self.received_messages["/test/0/aggregated/min_3min"][24][1]), 33.0, delta=0.1)
        self.assertAlmostEqual(float(self.received_messages["/test/0/aggregated/max_3min"][24][1]), 35.0, delta=0.1)


if __name__ == '__main__':
    unittest.main()




def get_schema():
    return {
            "data-preparation": {
                "description": "Root node for alcathous specific entries.",
                "type": "object",
                "properties": {
                    "no_data_behavior": {
                        "description": "How should the algorithm react in case of no data.",
                        "type": "string",
                        "enum": ["mute", "last_valid", "empty_message"]
                    },
                    "update_cycle": {
                        "type": "integer",
                        "minimum": 0,
                        "exclusiveMinimum": True,
                        "description": "new values published each ... seconds"
                    },
                    "number_worker": {
                        "type": "integer",
                        "minimum": 0,
                        "exclusiveMinimum": True,
                        "description": "how many worker threads should be spawned to process task queue"
                    },
                    "methods": {
                        "type": "array",
                        "items": {
                            "type": "object",
                            "properties": {
                                "name": {
                                    "description": "unique name for method",
                                    "type": "string"
                                },
                                "topic-pub-suffix": {
                                    "type": "string",
                                    "description": "results are publish to 'datapoints_prefix & topic-pub-suffix'."
                                },
                                "algorithm": {
                                    "type": "string",
                                    "enum": ["avg", "wavg", "count", "min", "max"]
                                },
                                "time_window": {
                                    "description": "use the values from the last ... minutes",
                                    "type": "number",
                                    "minimum": 0,
                                    "exclusiveMinimum": True
                                }
                            },
                            "required": ["time_window", "algorithm", "topic-pub-suffix"],
                            "additionalProperties": False
                        },
                        "additionalProperties": False
                    },
                    "datapoints": {
                        "type": "array",
                        "items": {
                            "type": "object",
                            "properties": {
                                "topic-sub": {
                                    "description": "subscribe to this topic and apply the methods to it",
                                    "type": "string"
                                },
                                "topic-pub-prefix": {
                                    "description": "publish results to this topic",
                                    "type": "string"
                                },
                                "zero_is_valid": {
                                    "description": "0 is valid or rejected",
                                    "type": "boolean"
                                },
                                "methods": {
                                    "description": "comma separated list of one or more strings. each entry must be "
                                                   "represented by a method entry.",
                                    "type": "string",
                                    "pattern": "(\\d+)(,\\s*\\d+)*"
                                }
                            },
                            "required": ["topic-sub", "topic-pub-prefix", "zero_is_valid", "methods"],
                            "additionalProperties": False
                        },
                        "additionalProperties": False
                    }
                },
                "required": ["no_data_behavior", "update_cycle", "number_worker", "methods", "datapoints"],
                "additionalProperties": False
            }
           }


import unittest
import os
import sys
sys.path.insert(0, os.path.abspath(os.path.join(os.path.dirname(__file__), '..')))
from alcathous.nodatabehavior import NoDataBehavior


class TestNoDataBehaviorEnum(unittest.TestCase):
    def test_enum(self):
        self.assertEqual(NoDataBehavior.LAST_VALID, NoDataBehavior.get_enum("last_valid"))
        self.assertEqual(NoDataBehavior.LAST_VALID, NoDataBehavior.get_enum("LAST_VALID"))
        self.assertEqual(NoDataBehavior.EMPTY_MESSAGE, NoDataBehavior.get_enum("EMPTY_MESSAGE"))
        self.assertEqual(NoDataBehavior.MUTE, NoDataBehavior.get_enum("mute"))
        self.assertRaises(ValueError, NoDataBehavior.get_enum, "")
        self.assertRaises(ValueError, NoDataBehavior.get_enum, "lastvalid")

if __name__ == '__main__':
    unittest.main()



import collections
import time


def generate_test_data(number_entries=100, time_interval=30):
    end_time = time.time()
    data_set = collections.OrderedDict()
    for t in range(number_entries, 0, -1):
        t = t - 1
        curr_time = end_time - (t * time_interval)
        data_set[curr_time] = t
    return end_time, data_set



from alcathous.algorithms.average import Average
from alcathous.algorithms.weightedaverage import WeightedAverage
from alcathous.algorithms.count import Count
from alcathous.algorithms.minimum import Minimum
from alcathous.algorithms.maximum import Maximum
from pelops import mylogger


class AlgorithmFactory:
    """
    Create algorithm instances based on the given yaml config structure.

    Note: each implementation of abstractalgorithm must be represented by a class variable. E.g. the implementation
    'Average' is added with the line 'avg = Average'. 'avg' represents the identifier that must be used in the
    config yaml.
    """

    avg = Average  # used by the line "klass = cls.__dict__[class_name]"
    wavg = WeightedAverage  # used by the line "klass = cls.__dict__[class_name]"
    count = Count  # used by the line "klass = cls.__dict__[class_name]"
    min = Minimum  # used by the line "klass = cls.__dict__[class_name]"
    max = Maximum  # used by the line "klass = cls.__dict__[class_name]"

    @classmethod
    def get_instance(cls, name, config, parent_logger, data_set, lock_data_set, topic_pub_prefix, mqtt_client,
                     no_data_behavior):
        """
        Generate a method / algorithm instance.

        :param name: Name of the to be created method / algorithm instance.
        :param config: Configuration yaml containing config for all algorithms.
        :param parent_logger: logger instance from the parent.
        :param data_set: the data the algortihm should work with
        :param lock_data_set: the algorithm works with the data set only if a lock has been acquired successfully
        :param topic_pub_prefix: prefix+name is the topic the result will be published to
        :param mqtt_client: instance of an mqtt client
        :param no_data_behavior: defines how the algorithm should react if no data is available.
        :return: The corresponding method / algorithm instance
        """
        _logger = mylogger.get_child(parent_logger, cls.__name__)
        class_name = str(config["algorithm"]).lower()
        try:
            klass = cls.__dict__[class_name]
        except KeyError:
            raise ValueError("Unknown value for AlgorithmFactory '{}'. Expected {}.".
                             format(class_name, cls.__dict__.keys()))
        _logger.info("AlgorithmFactory.get_instance - creating instance of '{}'.".format(klass.__name__))
        instance = klass(name, config, parent_logger, data_set, lock_data_set, topic_pub_prefix, mqtt_client,
                         no_data_behavior)
        return instance

    @classmethod
    def get_instances(cls, method_names, config_methods, parent_logger, data_set, lock_data_set, topic_pub_prefix,
                      mqtt_client, no_data_behavior):
        """
        Take the list of method names and generates the corresponding algorithm instances.

        :param method_names: List of algorithms that should be generated.
        :param config_methods: Configuration yaml containing config for all algorithms.
        :param parent_logger: logger instance from the parent.
        :param data_set: the data the algortihm should work with
        :param lock_data_set: the algorithm works with the data set only if a lock has been acquired successfully
        :param topic_pub_prefix: prefix+name is the topic the result will be published to
        :param mqtt_client: instance of an mqtt client
        :param no_data_behavior: defines how the algorithm should react if no data is available.
        :return: List of methods / algorithm instances
        """
        _logger = mylogger.get_child(parent_logger, cls.__name__)
        _logger.info("AlgorithmFactory.get_instances - creating instances of '{}'.".format(method_names))
        _logger.debug("AlgorithmFactory.get_instances - config_methods '{}'.".format(config_methods))
        methods = []
        for name in method_names:
            try:
                config = config_methods[name]
            except KeyError:
                raise ValueError("Unknown value for method '{}'. Expected {}.".
                                 format(name, config_methods.keys()))
            m = AlgorithmFactory.get_instance(name, config, parent_logger, data_set, lock_data_set,
                                              topic_pub_prefix, mqtt_client, no_data_behavior)
            methods.append(m)
        _logger.info("AlgorithmFactory.get_instances - created {} instances.".format(len(methods)))
        return methods



from pelops.abstractmicroservice import AbstractMicroservice
import time
import threading
from alcathous.datapoint import DataPoint
from alcathous.nodatabehavior import NoDataBehavior
from alcathous.worker import Worker
import alcathous.schema as schema


class DataPointManager(AbstractMicroservice):
    """
    Manage the data points by creating/start/stopping them and invocation of the process methods of the algorithms
    regularly.

    Config yaml:
    mqtt:
        mqtt-address: localhost
        mqtt-port: 1883
        mqtt-credentials: ~/credentials.yaml
        log-level: INFO

    logger:
        log-level: DEBUG
        log-file: alcathous.log

    data-preparation:  # alcathous root node
        no_data_behavior: last_valid  # mute, last_valid, empty_message
        update_cycle: 30  # new values published each ... seconds
        number_worker: 2  # how many worker threads should be spawned to process task queue

        methods:
            avg_5min:
                topic-pub-suffix: avg_5min
                algorithm: avg  # avg - average, wavg - weighted average, count, min, max
                time_window: 5  # use the values from the last ... minutes

            wavg_5min:
                topic-pub-suffix: wavg_5min
                algorithm: wavg  # avg - average, wavg - weighted average, count, min, max
                time_window: 5  # use the values from the last ... minutes

        datapoints:
            - topic-sub: /test/0/raw
              topic-pub-prefix: /test/0/aggregated/
              zero_is_valid: False  # 0 is valid or rejected
              methods: wavg_5min, avg_5min
    """

    _data_points = None  # list of datapoint instances
    _processes = None  # list with references to all process functions of the algorithms of the datapoints
    _purges = None  # list with references to all purge functions of the datapoints
    _no_data_behavior = None  # defines how the algorithm should react if no data is available.
    _update_cycle = None  # call all processes every n seconds
    _worker = None  # instance of the worker manager
    _do_loop_thread = None  # contains the loop method thread

    def __init__(self, config, mqtt_client=None, logger=None):
        """
        Constructor.

        :param config: config yaml structure
        :param mqtt_client: instance of an mqtt client (optional)
        :param logger: instance of a logger (optional)
        """

        AbstractMicroservice.__init__(self, config, "data-preparation", mqtt_client=mqtt_client, logger=logger)

        self._purges = []
        self._processes = []
        self._data_points = []

        self._update_cycle = int(self._config["update_cycle"])
        self._no_data_behavior = NoDataBehavior.get_enum(str(self._config["no_data_behavior"]))

        self._worker = Worker(self._config["number_worker"], self._logger)
        self._do_loop_thread = threading.Thread(target=self._do_loop)

        _config_methods = {}
        for m in self._config["methods"]:
            key = m["name"]
            if key in _config_methods:
                self._logger.error("DataPointManager - method name must be unique ({}).".format(key))
                raise ValueError("DataPointManager - method name must be unique ({}).".format(key))
            _config_methods[key] = m

        for config_data_point in self._config["datapoints"]:
            dp = DataPoint(config_data_point, _config_methods, self._logger, self._mqtt_client,
                           self._no_data_behavior)
            self._purges.append(dp.purge_old_values)
            for method in dp.methods:
                process = method.process
                cost = method.execution_points_estimation()
                self._logger.info("DataPointManager - adding process '{}' with cost '{}'.".
                                  format(process.__name__, cost))
                self._processes.append((process, cost))
            self._data_points.append(dp)

        self._processes.sort(key=lambda tup: tup[1], reverse=True)  # sort processes by their cost most expensive first

    def _get_description(self):
        return "This software subscribes to mqtt-topics that contain raw sensor data and publishes average values " \
               "for configurable time spans."

    def _loop_process(self):
        """
        Call all process and purge methods.
        """

        timestamp = time.time()
        self._logger.info("DataPointManager - started work for timestamp '{} s'.".format(timestamp))
        for p in self._processes:
            self._worker.queue.put((p[0], timestamp))
            self._logger.info("DataPointManager - waiting for worker to finish processing the algorithms.")
        self._worker.queue.join()
        for p in self._purges:
            self._worker.queue.put((p, timestamp))
        self._logger.info("DataPointManager - waiting for worker to purge outdated values.")
        self._worker.queue.join()

    def _do_loop(self):
        """
        Call _loop_process every n seconds.
        """
        self._logger.info("DataPointManager - start loop.")
        while not self._stop_service.is_set():
            start = time.time()
            self._loop_process()
            sleep_for = max(0, self._update_cycle - (time.time() - start))
            self._logger.info("DataPointManager - wait for '{} s'.".format(sleep_for))
            self._stop_service.wait(sleep_for)
        self._logger.info("DataPointManager - exited loop.")

    @classmethod
    def _get_schema(cls):
        """
        Get the sub schema to validate the yaml-config file against.

        :return: json-schema dict
        """
        return schema.get_schema()

    def _start(self):
        """
        Start all datapoints, worker and the process loop.
        """
        for dp in self._data_points:
            dp.start()
        self._worker.start()
        self._do_loop_thread.start()

    def _stop(self):
        """
        Stop all datapoints, worker and the process loop.
        """
        for dp in self._data_points:
            dp.stop()
        self._do_loop_thread.join()
        self._worker.stop()


def standalone():
    """Calls the static method DataPointManager.standalone()."""
    DataPointManager.standalone()


if __name__ == "__main__":
    DataPointManager.standalone()



import unittest
import os
import sys
sys.path.insert(0, os.path.abspath(os.path.join(os.path.dirname(__file__), '..')))
from threading import Lock, Event
import tests.testdatagenerator as testdatagenerator
from pelops import mylogger, mqttclient, read_config
from alcathous.algorithms.average import Average
from alcathous.algorithms.weightedaverage import WeightedAverage
from alcathous.algorithms.count import Count
from alcathous.algorithms.maximum import Maximum
from alcathous.algorithms.minimum import Minimum
from alcathous.nodatabehavior import NoDataBehavior


class Tools:
    def _on_response(self, value):
        self.last_response = value.decode("utf-8")
        self.on_response_sync.set()

    def _wait_for_response(self, message):
        if not self.on_response_sync.wait(1):
            self.fail("Timeout waiting on below response for message '{}'.".format(message))
        self.on_response_sync.clear()

    def _setUp(self):
        self.mqtt_client = mqttclient.MQTTClient(self.config["mqtt"], self.logger, True)
        self.mqtt_client.connect()
        self.lock_data_set = Lock()
        self.end_time, self.data_set = testdatagenerator.generate_test_data(number_entries=100, time_interval=30)
        self.on_response_sync = Event()
        self.on_response_sync.clear()
        self.last_response = None
        self.pos = 0
        try:
            for m in self.config["data-preparation"]["methods"]:
                if m["name"] == self.name:
                    self.config_alg = m
            self.pub_prefix = self.config["data-preparation"]["datapoints"][self.pos]["topic-pub-prefix"]
            self.pub_suffix = self.config_alg["topic-pub-suffix"]
            self.no_data_behavior = NoDataBehavior.get_enum(self.config["data-preparation"]["no_data_behavior"])
        except KeyError:
            raise ValueError("config malformed.")

    def _tearDown(self):
        self.mqtt_client.disconnect()


class TestAlgorithmAvg(unittest.TestCase, Tools):
    @classmethod
    def setUpClass(cls):
        cls.config = read_config.read_config(
            os.path.abspath(os.path.join(os.path.dirname(__file__), '..'))+"/tests/config.yaml")
        cls.logger = mylogger.create_logger(cls.config["logger"], __name__)
        cls.logger.info("TestAlgorithmAvg - start")

    @classmethod
    def tearDownClass(cls):
        cls.logger.info("TestAlgorithmAvg - stop")

    def setUp(self):
        self.name = "avg_5min"
        self._setUp()

    def tearDown(self):
        self._tearDown()

    def test_avg0_init(self):
        alg = Average(self.name, self.config_alg, self.logger, self.data_set, self.lock_data_set, self.pub_prefix,
                      self.mqtt_client, self.no_data_behavior)
        self.assertEqual(alg.name, self.name)
        self.assertEqual(alg._time_window, 300)
        self.assertEqual(len(alg._data_set), 100)
        self.assertEqual(alg._no_data_behavior, NoDataBehavior.LAST_VALID)
        self.assertEqual(alg._last_valid, None)
        self.assertEqual(alg._topic, self.pub_prefix + self.pub_suffix)
        self.assertIsNotNone(alg._mqtt_client)
        self.assertIsNotNone(alg._logger)
        self.assertEqual(alg._config, self.config_alg)

    def test_avg1_execution_points_estimation(self):
        alg = Average(self.name, self.config_alg, self.logger, self.data_set, self.lock_data_set, self.pub_prefix,
                      self.mqtt_client, self.no_data_behavior)
        self.assertEqual(alg.execution_points_estimation(), 300)

    def test_avg2_process(self):
        alg = Average(self.name, self.config_alg, self.logger, self.data_set, self.lock_data_set, self.pub_prefix,
                      self.mqtt_client, self.no_data_behavior)
        self.mqtt_client.subscribe(alg._topic, self._on_response)
        alg.process(self.end_time)
        self._wait_for_response("process average")
        self.assertIsNotNone(self.last_response)
        self.assertEqual(alg._last_valid, 5.0)
        self.assertEqual(float(self.last_response), 5.0)


class TestAlgorithmWAvg(unittest.TestCase, Tools):
    @classmethod
    def setUpClass(cls):
        cls.config = read_config.read_config(
            os.path.abspath(os.path.join(os.path.dirname(__file__), '..'))+"/tests/config.yaml")
        cls.logger = mylogger.create_logger(cls.config["logger"], __name__)
        cls.logger.info("TestAlgorithmWAvg - start")

    @classmethod
    def tearDownClass(cls):
        cls.logger.info("TestAlgorithmWAvg - stop")

    def setUp(self):
        self.name = "wavg_5min"
        self._setUp()

    def tearDown(self):
        self._tearDown()

    def test_wavg0_init(self):
        alg = WeightedAverage(self.name, self.config_alg, self.logger, self.data_set, self.lock_data_set,
                              self.pub_prefix, self.mqtt_client, self.no_data_behavior)
        self.assertEqual(alg.name, self.name)
        self.assertEqual(alg._time_window, 300)
        self.assertEqual(len(alg._data_set), 100)
        self.assertEqual(alg._no_data_behavior, NoDataBehavior.LAST_VALID)
        self.assertEqual(alg._last_valid, None)
        self.assertEqual(alg._topic, self.pub_prefix + self.pub_suffix)
        self.assertIsNotNone(alg._mqtt_client)
        self.assertIsNotNone(alg._logger)
        self.assertEqual(alg._config, self.config_alg)

    def test_wavg1_execution_points_estimation(self):
        alg = WeightedAverage(self.name, self.config_alg, self.logger, self.data_set, self.lock_data_set,
                              self.pub_prefix, self.mqtt_client, self.no_data_behavior)
        self.assertEqual(alg.execution_points_estimation(), 330)

    def test_wavg2_process(self):
        alg = WeightedAverage(self.name, self.config_alg, self.logger, self.data_set, self.lock_data_set,
                              self.pub_prefix, self.mqtt_client, self.no_data_behavior)
        self.mqtt_client.subscribe(alg._topic, self._on_response)
        alg.process(self.end_time)
        self._wait_for_response("process weighted average")
        self.assertIsNotNone(self.last_response)
        self.assertEqual(alg._last_valid, 3.0)
        self.assertEqual(float(self.last_response), 3.0)


class TestAlgorithmCount(unittest.TestCase, Tools):
    @classmethod
    def setUpClass(cls):
        cls.config = read_config.read_config(
            os.path.abspath(os.path.join(os.path.dirname(__file__), '..'))+"/tests/config.yaml")
        cls.logger = mylogger.create_logger(cls.config["logger"], __name__)
        cls.logger.info("TestAlgorithmCount - start")

    @classmethod
    def tearDownClass(cls):
        cls.logger.info("TestAlgorithmCount - stop")

    def setUp(self):
        self.name = "count_2min"
        self._setUp()

    def tearDown(self):
        self._tearDown()

    def test_count0_init(self):
        alg = Count(self.name, self.config_alg, self.logger, self.data_set, self.lock_data_set,
                    self.pub_prefix, self.mqtt_client, self.no_data_behavior)
        self.assertEqual(alg.name, self.name)
        self.assertEqual(alg._time_window, 120)
        self.assertEqual(len(alg._data_set), 100)
        self.assertEqual(alg._no_data_behavior, NoDataBehavior.LAST_VALID)
        self.assertEqual(alg._last_valid, None)
        self.assertEqual(alg._topic, self.pub_prefix + self.pub_suffix)
        self.assertIsNotNone(alg._mqtt_client)
        self.assertIsNotNone(alg._logger)
        self.assertEqual(alg._config, self.config_alg)

    def test_count1_execution_points_estimation(self):
        alg = Count(self.name, self.config_alg, self.logger, self.data_set, self.lock_data_set,
                    self.pub_prefix, self.mqtt_client, self.no_data_behavior)
        self.assertEqual(alg.execution_points_estimation(), 108)

    def test_count2_process(self):
        alg = Count(self.name, self.config_alg, self.logger, self.data_set, self.lock_data_set,
                    self.pub_prefix, self.mqtt_client, self.no_data_behavior)
        self.mqtt_client.subscribe(alg._topic, self._on_response)
        alg.process(self.end_time)
        self._wait_for_response("process count")
        self.assertIsNotNone(self.last_response)
        self.assertEqual(alg._last_valid, 5)
        self.assertEqual(int(self.last_response), 5)


class TestAlgorithmMin(unittest.TestCase, Tools):
    @classmethod
    def setUpClass(cls):
        cls.config = read_config.read_config(
            os.path.abspath(os.path.join(os.path.dirname(__file__), '..'))+"/tests/config.yaml")
        cls.logger = mylogger.create_logger(cls.config["logger"], __name__)
        cls.logger.info("TestAlgorithmMin - start")

    @classmethod
    def tearDownClass(cls):
        cls.logger.info("TestAlgorithmMin - stop")

    def setUp(self):
        self.name = "min_3min"
        self._setUp()

    def tearDown(self):
        self._tearDown()

    def test_min0_init(self):
        alg = Minimum(self.name, self.config_alg, self.logger, self.data_set, self.lock_data_set,
                      self.pub_prefix, self.mqtt_client, self.no_data_behavior)
        self.assertEqual(alg.name, self.name)
        self.assertEqual(alg._time_window, 180)
        self.assertEqual(len(alg._data_set), 100)
        self.assertEqual(alg._no_data_behavior, NoDataBehavior.LAST_VALID)
        self.assertEqual(alg._last_valid, None)
        self.assertEqual(alg._topic, self.pub_prefix + self.pub_suffix)
        self.assertIsNotNone(alg._mqtt_client)
        self.assertIsNotNone(alg._logger)
        self.assertEqual(alg._config, self.config_alg)

    def test_min1_execution_points_estimation(self):
        alg = Minimum(self.name, self.config_alg, self.logger, self.data_set, self.lock_data_set,
                      self.pub_prefix, self.mqtt_client, self.no_data_behavior)
        self.assertEqual(alg.execution_points_estimation(), 180)

    def test_min2_process(self):
        alg = Minimum(self.name, self.config_alg, self.logger, self.data_set, self.lock_data_set,
                      self.pub_prefix, self.mqtt_client, self.no_data_behavior)
        self.mqtt_client.subscribe(alg._topic, self._on_response)
        alg.process(self.end_time)
        self._wait_for_response("process min")
        self.assertIsNotNone(self.last_response)
        self.assertEqual(alg._last_valid, 0)
        self.assertEqual(float(self.last_response), 0)


class TestAlgorithmMax(unittest.TestCase, Tools):
    @classmethod
    def setUpClass(cls):
        cls.config = read_config.read_config(
            os.path.abspath(os.path.join(os.path.dirname(__file__), '..'))+"/tests/config.yaml")
        cls.logger = mylogger.create_logger(cls.config["logger"], __name__)
        cls.logger.info("TestAlgorithmMax - start")

    @classmethod
    def tearDownClass(cls):
        cls.logger.info("TestAlgorithmMax - stop")

    def setUp(self):
        self.name = "max_3min"
        self._setUp()

    def tearDown(self):
        self._tearDown()

    def test_max0_init(self):
        alg = Maximum(self.name, self.config_alg, self.logger, self.data_set, self.lock_data_set,
                      self.pub_prefix, self.mqtt_client, self.no_data_behavior)
        self.assertEqual(alg.name, self.name)
        self.assertEqual(alg._time_window, 180)
        self.assertEqual(len(alg._data_set), 100)
        self.assertEqual(alg._no_data_behavior, NoDataBehavior.LAST_VALID)
        self.assertEqual(alg._last_valid, None)
        self.assertEqual(alg._topic, self.pub_prefix + self.pub_suffix)
        self.assertIsNotNone(alg._mqtt_client)
        self.assertIsNotNone(alg._logger)
        self.assertEqual(alg._config, self.config_alg)

    def test_max1_execution_points_estimation(self):
        alg = Maximum(self.name, self.config_alg, self.logger, self.data_set, self.lock_data_set,
                      self.pub_prefix, self.mqtt_client, self.no_data_behavior)
        self.assertEqual(alg.execution_points_estimation(), 180)

    def test_max2_process(self):
        alg = Maximum(self.name, self.config_alg, self.logger, self.data_set, self.lock_data_set,
                      self.pub_prefix, self.mqtt_client, self.no_data_behavior)
        self.mqtt_client.subscribe(alg._topic, self._on_response)
        alg.process(self.end_time)
        self._wait_for_response("process max")
        self.assertIsNotNone(self.last_response)
        self.assertEqual(alg._last_valid, 6)
        self.assertEqual(float(self.last_response), 6)


if __name__ == '__main__':
    unittest.main()
